import java.io.*;
import java.util.ArrayList;

public class Spelling {
    private SortedArrayList<String> list;

    public class Word {
        private String word;
        private boolean isCorrect;
        private String nearToWord;

        public Word(String word, boolean isCorrect, String nearToWord) {
            this.word = word;
            this.isCorrect = isCorrect;
            this.nearToWord = nearToWord;
        }

        public String nearToWord() {
            return nearToWord;
        }

        public boolean isCorrect() {
            return isCorrect;
        }

        public String toString() {
            return word;
        }
    }

    public Spelling(Reader reader) throws IOException {
        list = new SortedArrayList<String>();

        BufferedReader br = new BufferedReader(reader);

        String word;

        while ((word = br.readLine()) != null) {
            if (!word.startsWith("%")) {
                list.add(word.toLowerCase());
            }
        }

        reader.close();
    }

    public boolean checkWord(String word, StringBuilder nearToWord) {
        int index = list.binarySearch(word.toLowerCase());

        if (index < 0) {
            nearToWord.append(~index < list.size() ? list.get(~index) : "");
            return false;
        }
        return true;
    }

    public Iterable<Word> checkText(String text) {
        String[] tokens = text.split("[ .,:;\r\n]+");

        var result = new ArrayList<Word>();

        for (String token : tokens) {

            String nearToWord = "";
            int index = list.binarySearch(token.toLowerCase());

            if (index < 0)
                nearToWord = ~index < list.size() ? list.get(~index) : "";

            result.add(new Word(token, index >= 0, nearToWord));
        }

        return result;
    }
}